#!/usr/bin/env python
# coding: utf-8
import asyncio
import logging

import uvicorn
from fastapi import FastAPI, HTTPException, Request, Body

from Article import Article
from constants import CONNECTION_STRING, MAX_SUMMARIES_PER_USER
from summarizer.Summarizer import Summarizer

app = FastAPI()
summarizer = Summarizer()

from motor.motor_asyncio import AsyncIOMotorClient


async def get_database():
    # Create a connection pool
    client = AsyncIOMotorClient(CONNECTION_STRING)

    # Return the database object
    return client['user_shopping_list']


def get_token(request):
    return request.headers.get("Authorization")


@app.get("/api/articles")
async def read_last_articles(request: Request):
    token = get_token(request)

    db = await get_database()

    # Search the database for the collection with the matching token
    collection = await db.collections.find_one({"token": token})
    if collection is None:
        raise HTTPException(status_code=404, detail="Collection not found")

    # Return the articles from the collection
    return {"articles": collection["articles"]}


@app.post("/api/summarize")
async def summarize(request: Request, article: Article = Body(..., validate=True)):
    # try:
    # Get the Authorization header and extract the token
    token = get_token(request)
    print(token)

    db = await get_database()

    # Generate the summary of the text
    summary = summarizer.generate_summary(article.text, 2)

    # Check if the token exists in the database
    collection = await db.collections.find_one({"token": token})
    if collection is None:
        # If the token does not exist, create a new collection for it
        collection = {
            "token": token,
            "articles": [summary]
        }
        await db.collections.insert_one(collection)
    else:
        # If the token exists, add the new article to the collection
        articles = collection["articles"]
        logging.info(f'articles before update: {articles}')
        if len(articles) == MAX_SUMMARIES_PER_USER:
            # If there are already 5 articles, delete the oldest one
            articles.pop(0)
        articles.append(summary)
        await db.collections.update_one({"token": token}, {"$set": {"articles": articles}})
    return {"summary": summary}


# except ValidationError as e:
#     return {"error": str(e)}


if __name__ == "__main__":
    asyncio.run(uvicorn.run(app, host="127.0.0.1", port=8000))
